﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

void main()
{
	float	fBrightness = params[0];

	if ( args[A_TW] <= 0.0 ) {
		float	b = abs( step( fBrightness, 0.0 ) - max( gl_Color.r, max( gl_Color.g, gl_Color.b ) ) );

		if ( b <= abs( fBrightness ) ) {
			discard;
		}

		gl_FragColor = gl_Color;
		return;
	}

	vec4	Pixel = getTexColor( gl_TexCoord[0].st );

	float	b = abs( step( fBrightness, 0.0 ) - max( Pixel.r, max( Pixel.g, Pixel.b ) ) );

	if ( b <= abs( fBrightness ) ) {
		discard;
	}

	vec4	Blend = getBlendColor( Pixel );

	gl_FragColor = Blend;
}
