﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_U1	= 2;
const int	A_V1	= 3;
const int	A_LU	= 4;
const int	A_TV	= 5;
const int	A_CU	= 6;
const int	A_CV	= 7;
const int	A_RU	= 8;
const int	A_BV	= 9;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

void main()
{
	float	fPhase = params[0];
	float	fDirection = params[1];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	float	PI = 3.14159265358979;
	float	e = 1.0e-10;
	vec2	uv1 = vec2( args[A_U1] + e, args[A_V1] + e );
	vec2	tx = gl_TexCoord[0].st / uv1;
	vec2	c = vec2( args[A_CU], args[A_CV] ) / uv1;
	vec2	lt = vec2( args[A_LU], args[A_TV] ) / uv1;
	vec2	rb = vec2( args[A_RU], args[A_BV] ) / uv1;
	vec2	d = rb - lt;
	vec2	v = tx - c;
	float	r = min( abs( rb.x - c.x ), abs( rb.y - c.y ) ) + e;
	float	dis = length( v );

	if ( dis > r ) {
		discard;
	}

	vec2	nv = normalize( v );
	float	uu = 1.0 - dis / r;
	float	vv = ( atan( nv.y, nv.x ) / PI + 1.0 ) * 0.5 + fPhase;

	if ( vv < 0.0 ) {
		vv += 1.0;
	}
	if ( vv > 1.0 ) {
		vv -= 1.0;
	}

	if ( fDirection <= 0.0 ) {
		vv = 1.0 - vv;
	}

	vec2	st = d * vec2( uu, vv ) * uv1;

	vec4	Pixel = getTexColor( vec2( args[A_LU], args[A_TV] ) + st );
	vec4	Blend = getBlendColor( Pixel );

	gl_FragColor = Blend;
}
