﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

vec4 toHSB( vec4 color )
{
	vec4	K = vec4( 0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0 );
	vec4	p = mix( vec4( color.bg, K.wz ), vec4( color.gb, K.xy ), step( color.b, color.g ) );
	vec4	q = mix( vec4( p.xyw, color.r ), vec4( color.r, p.yzx ), step( p.x, color.r ) );

	float	d = q.x - min( q.w, q.y );
	float	e = 1.0e-10;

	return	vec4( abs( q.z + ( q.w - q.y ) / ( 6.0 * d + e ) ), d / ( q.x + e ), q.x, color.a );
}

vec4 toRGB( vec4 hsb )
{
	vec4	K = vec4( 1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0 );
	vec3	p = abs( fract( hsb.xxx + K.xyz ) * 6.0 - K.www );

	return	vec4( hsb.z * mix( K.xxx, clamp( p - K.xxx, 0.0, 1.0 ), hsb.y ), hsb.w );
}

vec4 shiftHSB( vec4 hsb, float fRatioH, float fRatioS, float fRatioB )
{
	vec4	shift = hsb;

	shift.x += fRatioH;
	shift.y = clamp( hsb.y + fRatioS, 0.0, 1.0 );
	shift.z = clamp( hsb.z + fRatioB, 0.0, 1.0 );

	if ( shift.x < 0.0 ) {
		shift.x += 1.0;
	}
	if ( shift.x > 1.0 ) {
		shift.x -= 1.0;
	}

	return	shift;
}

vec4 adjustHSB( vec4 color, float fRatioH, float fRatioS, float fRatioB )
{
	vec4	hsb;

	hsb = toHSB( color );

	hsb = shiftHSB( hsb, fRatioH, fRatioS, fRatioB );

	return	toRGB( hsb );
}

void main()
{
	float	fHue = params[0];
	float	fSaturation = params[1];
	float	fBrightness = params[2];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = adjustHSB( gl_Color, fHue, fSaturation, fBrightness );
		return;
	}

	vec4	Pixel = getTexColor( gl_TexCoord[0].st );
	vec4	Blend = getBlendColor( adjustHSB( Pixel, fHue, fSaturation, fBrightness ) );

	gl_FragColor = Blend;
}
