﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_U1	= 2;
const int	A_V1	= 3;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

void main()
{
	float	fDistance = params[0];
	float	fDirection = params[1];
	float	fPower = params[2];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	vec2	Coord;
	float	fPixW;
	float	fPixH;
	int		iCount;
	vec2	Vel;
	vec4	Pixel;
	float	PI = 3.14159265358979;

	Coord = gl_TexCoord[0].st;

	fPixW = args[A_U1];
	fPixH = args[A_V1];

	iCount = int( floor( abs( fDistance ) * 96.0 ) );

	Vel = vec2( sin( fDirection * PI ) * fPixW, cos( fDirection * PI ) * fPixH ) * sign( fDistance );

	Coord += Vel * iCount;

	Coord -= Vel;
	Pixel = getTexColor( Coord );

	for ( int i = 1; i < iCount; i++ ) {
		Coord -= Vel;
		Pixel = mix( getTexColor( Coord ), Pixel, 0.96 * abs( fPower ) );
	}

	vec4	Blend = getBlendColor( Pixel );

	gl_FragColor = Blend;
}
