﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

float rand( vec2 p )
{
	return	fract( sin( dot( p, vec2( 12.9898, 78.233 ) ) ) * 43758.5453 );
}

void main()
{
	float	fPower = params[0];
	float	fColor = params[1];
	float	fPhase = params[2];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	vec2	Coord;
	vec2	u;
	vec4	Pixel;

	Coord = gl_TexCoord[0].st;
	u = vec2( floor( Coord.x * args[A_TW] ) / args[A_TW], floor( Coord.y * args[A_TH] ) / args[A_TH] );

	Pixel = getTexColor( Coord );

	vec2	t = u + vec2( fPhase );
	vec3	c;

	c.r = rand( t );
	c.g = mix( c.r, rand( t * c.r ), fColor );
	c.b = mix( c.g, rand( t * c.g ), fColor );

	Pixel.rgb = clamp( mix( Pixel.rgb, c.rgb, fPower ), 0.0, 1.0 );

	vec4	Blend = getBlendColor( Pixel );

	gl_FragColor = Blend;
}
