﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_U1	= 2;
const int	A_V1	= 3;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

float toOutlineValue( vec2 p, vec2 v, float fRatio )
{
	vec4	Pixel;
	float	lo;
	float	hi;
	float	e = 1.0e-5;

	lo = 0.0;

	Pixel = getTexColor( p + vec2( -v.x, 0.0 ) );
	lo += step( Pixel.a, fRatio );
	Pixel = getTexColor( p + vec2( +v.x, 0.0 ) );
	lo += step( Pixel.a, fRatio );
	Pixel = getTexColor( p + vec2( 0.0, -v.y ) );
	lo += step( Pixel.a, fRatio );
	Pixel = getTexColor( p + vec2( 0.0, +v.y ) );
	lo += step( Pixel.a, fRatio );

	Pixel = getTexColor( p );

	hi = step( fRatio + e, Pixel.a );

	return	min( hi * lo, 1.0 );
}

void main()
{
	float	fThreshold = params[0];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	vec2	Coord;
	float	fPixW;
	float	fPixH;

	Coord = gl_TexCoord[0].st;

	fPixW = args[A_U1];
	fPixH = args[A_V1];

	float	v = toOutlineValue( Coord, vec2( fPixW, fPixH ), abs( fThreshold ) );

	if ( v <= 0.0 ) {
		discard;
	}

	vec4	Blend = getBlendColor( vec4( v ) );

	gl_FragColor = Blend;
}
