﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

void main()
{
	float	fPower = params[0];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	float	e = 1.0e-10;
	float	w = args[A_TW] + e;
	float	h = args[A_TH] + e;
	float	v = 1.0 + 96.0 * fPower;
	vec2	Coord;
	vec4	Pixel;

	Coord = vec2( floor( gl_TexCoord[0].s * w / v ) * v / w, floor( gl_TexCoord[0].t * h / v ) * v / h );

	Pixel = getTexColor( Coord.st );

	vec4	Blend = getBlendColor( Pixel );

	gl_FragColor = Blend;
}
