﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_U1	= 2;
const int	A_V1	= 3;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

float rand( vec2 p )
{
	return	fract( sin( dot( p, vec2( 12.9898, 78.233 ) ) ) * 43758.5453 );
}

void main()
{
	float	fPower = params[0];
	float	fRatio = params[1];
	float	fPhase = params[2];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	float	PI = 3.14159265358979;
	vec2	Coord;
	vec2	u;

	Coord = gl_TexCoord[0].st;
	u = vec2( floor( Coord.x * args[A_TW] ) / args[A_TW], floor( Coord.y * args[A_TH] ) / args[A_TH] );

	vec2	t = u + vec2( fPhase );
	float	r = rand( t ) * PI * 2.0 - PI;

	float	r2 = step( rand( t * r ), fRatio );

	vec2	v = vec2( sin( r ), cos( r ) ) * vec2( args[A_U1], args[A_V1] ) * 96.0 * fPower * r2;

	vec4	Pixel = getTexColor( Coord + v );
	vec4	Blend = getBlendColor( Pixel );

	gl_FragColor = Blend;
}
