﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

float toGrayValue( vec4 color )
{
	float	fRatioR = 0.298912;
	float	fRatioG = 0.586611;
	float	fRatioB = 0.114478;
	vec3	grayScale = vec3( fRatioR, fRatioG, fRatioB );

	return	dot( color.rgb, grayScale );
}

vec4 toSepiaColor( vec4 color )
{
	float	fRatioR = 1.07;
	float	fRatioG = 0.74;
	float	fRatioB = 0.43;
	vec3	sepiaScale = vec3( fRatioR, fRatioG, fRatioB );

	return	vec4( color.rgb * sepiaScale, color.a );
}

void main()
{
	float	fPower = params[0];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	vec4	Pixel;
	vec4	Gray;
	vec4	Sepia;

	Pixel = getTexColor( gl_TexCoord[0].st );

	Gray = vec4( vec3( toGrayValue( Pixel ) ), Pixel.a );
	Sepia = mix( toSepiaColor( Gray ), Gray, step( fPower, 0.0 ) );

	vec4	Blend = getBlendColor( mix( Pixel, Sepia, abs( fPower ) ) );

	gl_FragColor = Blend;
}
