﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_U1	= 2;
const int	A_V1	= 3;
const int	A_LU	= 4;
const int	A_TV	= 5;
const int	A_CU	= 6;
const int	A_CV	= 7;
const int	A_RU	= 8;
const int	A_BV	= 9;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

void main()
{
	float	fPower = params[0];
	float	fDecay = params[1];
	float	fColor = params[2];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	float	e = 1.0e-10;
	float	p = abs( fPower );
	float	d = abs( fDecay );
	vec2	uv1 = vec2( args[A_U1] + e, args[A_V1] + e );
	vec2	c = vec2( args[A_CU], args[A_CV] ) / uv1;
	vec2	rb = vec2( args[A_RU], args[A_BV] ) / uv1;
	float	l = min( abs( rb.x - c.x ), abs( rb.y - c.y ) ) + e;
	vec2	t = gl_TexCoord[0].st / uv1;
	vec2	v = c - t;

	float	fR = max( p / ( length( v / l ) + e ) - p, 0.0 );
	float	fL = max( p - length( v / l ), 0.0 );

	fR = clamp( mix( fR, fL, d ), 0.0, 1.0 );

	vec4	Pixel = vec4( mix( vec3( 1.0 ), getTexColor( gl_TexCoord[0].st ).rgb, fColor ), fR );
	vec4	Blend = getBlendColor( Pixel );

	gl_FragColor = Blend;
}
