﻿// Copyright 2019 Web Technology Corp.
#version 120
uniform sampler2D	texture2d;
uniform float		args[16];
uniform float		params[16];

const int	A_TW	= 0;
const int	A_TH	= 1;
const int	A_PM	= 10;

vec4 getTexColor( vec2 c )
{
	vec4	p = texture2D( texture2d, c );

	if ( args[A_PM] <= 0.0 ) {
		return	p;
	}
	if ( p.a <= 0.0 ) {
		return	vec4( 0.0 );
	}

	return	vec4( p.rgb / p.a, p.a );
}

vec4 getBlendColor( vec4 p )
{
	return	vec4( p.rgb * gl_TexCoord[1].x + mix( vec3( 1.0 ), p.rgb, gl_TexCoord[1].z ) * gl_Color.rgb * gl_TexCoord[1].y, p.a * gl_Color.a );
}

void main()
{
	float	fThreshold = params[0];
	float	fStage = params[1];
	float	fColor = params[2];

	if ( args[A_TW] <= 0.0 ) {
		gl_FragColor = gl_Color;
		return;
	}

	vec4	Pixel;

	Pixel = getTexColor( gl_TexCoord[0].st );

	float	d = 1.0 + 255.0 * abs( fStage );
	float	e = 1.0e-10;
	float	t = abs( fThreshold );
	float	b = abs( step( fThreshold, 0.0 ) - max( Pixel.r, max( Pixel.g, Pixel.b ) ) );
	float	r = clamp( floor( ( b - t ) / ( t + e ) * 256.0 / d ) * d / 256.0, 0.0, 1.0 );
	float	c = step( t, b ) * r;
	vec3	v = mix( vec3( c ), Pixel.rgb * c, fColor );

	vec4	Blend = getBlendColor( vec4( v, Pixel.a ) );

	gl_FragColor = Blend;
}
